unit ElegantMDI;

{
ElegantMDI: this component is a new elegant realiztion of the
            old MDI interface. All windows (MDIChild) will be
            presented as buttons on autohidden panel. User can
            toggle between windows without choosing items in main
            menu. It looks very effectively.

Status: Freeware with full source.

Installation: Choose 'Install Component...' at the main menu.
              Choose the 'Into new package' tab. Select in
              the 'Unit file name' the elegantmdi.pas. And in the
              'Package file name' input any file name
              (not 'elegantmdi' - 'elegantmdipack', for example).
              Press 'Ok' button. That's all!

Working with 'Elegant MDI Interface'
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
This component are fully compatible with TToolBar. It has all their
properties, events and methods.

TElegantMDI have additional properties (relatively TToolBar).

properties
~~~~~~~~~~
Removable: Boolean - if set to True, then component will be remove and
           appear on the screen automaticly.

AnimateSpeed: Integer - is a speed of animation on removing or appearing
              panel. If set to 0 then panel remove or appear at once.

DelayOnAppearance: Integer - is a delay (ms.) on appearing of the panel when
                   mouse's pointer is over a component.

ImageIndexDocument: Integer - is an ImageIndex for panel's buttons.

---------- Other products ----------

MaxSpace
~~~~~~~~
MaxSpace just makes Delphi and C++ Builder more comfortable in usage.
It turns the IDE ToolBar and ObjectInspector into emerging state,
giving you unbelievable freedom of program editing and form designing.
For more information visit http://www.zecos.com/maxspc

WavePanel
~~~~~~~~~
WavePanel - component with running colors, like panel at the bottom
of the startup screen when Windows 9x loading. Freeware.

MinModal
~~~~~~~~
Minimizer for modal windows. Why we can't minimize an application when
a modal window is active? Drop this component to the form, open form
by method 'ShowModal' and try minimize this form at run time. All your
application will be minimized!

All this programs you can download from http://www.zecos.com

Zecos Software team
http://www.zecos.com
support@zecos.com
}

 {$WARN SYMBOL_DEPRECATED OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls;

type
  TElegantMDI = class(TToolBar)
  private
    Form: TForm;
    OldWndClientProc: Pointer;
    InstanceWndClientProc: Pointer;
    FImageIndexDocument: Integer;
    FRemovable: Boolean;
    FAnimateSpeed: Integer;
    FDelayOnAppearance: Integer;
    FAbout: String;
    FRealButtonWidth: Integer;
    procedure SetImageIndexDocument(const Value: Integer);
    procedure SetAnimateSpeed(const Value: Integer);
    procedure InitRemovable;
    procedure SetRemovable(const Value: Boolean);
    function GetHeight: Integer;
    function GetWidth: Integer;
    procedure SetDelayOnAppearance(const Value: Integer);
    procedure SetAbout(const Value: String);
  protected
    procedure WndClientProc(var Message: TMessage); virtual;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure WMTimer(var Msg: TMessage); message WM_TIMER;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property About: String read FAbout write SetAbout stored False;
    property ImageIndexDocument: Integer read FImageIndexDocument write SetImageIndexDocument;
    property Removable: Boolean read FRemovable write SetRemovable;
    property AnimateSpeed: Integer read FAnimateSpeed write SetAnimateSpeed;
    property DelayOnAppearance: Integer read FDelayOnAppearance write SetDelayOnAppearance;
  end;

procedure Register;

implementation


uses Menus, AboutComponent;

const
  Gap = 10;
  TimerID = 12;
  WebHome = 'http://www.zecos.com';

type
  TChild = class(TObject)
  private
    Handle: THandle;
    ToolBar: TElegantMDI;
    Button: TToolButton;
    OldChildWndProc: Pointer;
    InstanceChildWndProc: Pointer;
    Popup: TPopupMenu;
    procedure OnPopupMenu(Sender: TObject);
  protected
    procedure ChildWndProc(var Message: TMessage); virtual;
    procedure ClickActBut(Sender: TObject);
    procedure MinimizeMDIWindow(Sender: TObject);
    procedure CloseMDIWindow(Sender: TObject);
    procedure RestoreMDIWindow(Sender: TObject);
    procedure MaximizeMDIWindow(Sender: TObject);
  public
    constructor Create(AHandle: THandle; AToolBar: TElegantMDI);
    destructor Destroy; override;
  end;

procedure Register;
begin
  RegisterComponents('Samples', [TElegantMDI]);
end;

procedure RememberMe(Sender: TObject);
begin
  with TFormAboutMDI.Create(Application) do begin
    WavePanel5.Caption := WebHome;
    Caption := 'www.zecos.com';
    Left := (Screen.Width-Width) div 2;
    Top  := (Screen.Height-Height) div 2;
    ShowModal;
    Free;
  end;
end;

{ TElegantMDI }

procedure TElegantMDI.CMMouseEnter(var Msg: TMessage);
begin
  inherited;
  KillTimer(Handle, TimerID);
  SetTimer(Handle, TimerID, DelayOnAppearance, nil);
end;

procedure TElegantMDI.CMMouseLeave(var Msg: TMessage);
var
  PosMouse: TPoint;
  i: Integer;
begin
  inherited;
  KillTimer(Handle,TimerID);
  if (Removable) {and (ButtonCount>0)} then begin
    GetCursorPos(PosMouse);
    PosMouse := Parent.ScreenToClient(PosMouse);
    if (PosMouse.x<=Left) or (PosMouse.x>=(Left+Width)) or
       (PosMouse.y<=Top) or (PosMouse.y>=(Top+Height)) then begin
      for i:=0 to ButtonCount-1 do begin
        Buttons[i].Visible := False;
      end;
      case Align of
        alTop, alBottom: begin
          if Height<>Gap then begin
            if AnimateSpeed>0 then begin
              InitRemovable;
              while Height>Gap do begin
                Application.ProcessMessages;
                Height := Height - AnimateSpeed;
              end;
            end;
            Height:=Gap;
          end;
        end;
        alLeft, alRight: begin
          if Width<>Gap then begin
            if AnimateSpeed>0 then begin
              InitRemovable;
              while Width>Gap do begin
                Application.ProcessMessages;
                Width := Width - AnimateSpeed;
              end;
            end;
            Width:=Gap;
          end;
        end;
      end;
      ButtonWidth := FRealButtonWidth;
    end;
  end;
end;

constructor TElegantMDI.Create(AOwner: TComponent);
var
  Component: TComponent;
begin
  inherited Create(AOwner);
  if (csDesigning in ComponentState) then begin
    ShowCaptions := True;
    AutoSize := True;
    EdgeBorders := EdgeBorders + [ebTop, ebBottom, ebLeft, ebRight];
    DelayOnAppearance := 400;
    List := True;
    Flat := True;
    AnimateSpeed := 0;
    Removable := True;
  end;
  FAbout := 'Elegant MDI Interface ('+WebHome+')';

  if not(csDesigning in ComponentState) then begin
    Component := Owner;
    while Assigned(Component) and not (Component is TForm) do
      Component:= Component.Owner;
    if not Assigned(Component) then
      Exit;
    Form := Component as TForm;

    if Form.Handle=Form.ClientHandle then ; // without this don''t work :o??????

    if (not (csDesigning in ComponentState)) and (Form.FormStyle=fsMDIForm) then begin
      InstanceWndClientProc := MakeObjectInstance(WndClientProc);
      OldWndClientProc := Pointer(GetWindowLong(Form.ClientHandle, GWL_WNDPROC));
      SetWindowLong(Form.ClientHandle, GWL_WNDPROC, Longint(InstanceWndClientProc));
    end;
  end;
end;

function TElegantMDI.GetHeight: Integer;
var
  tmp: Integer;
begin
  tmp := 0;
  if not Flat then
    tmp:=tmp+1;

  if ebBottom in EdgeBorders then
    tmp:=tmp+1;

  if ebTop in EdgeBorders then
    tmp:=tmp+1;

  Result := ButtonHeight + tmp*2 + BorderWidth*4;
end;

function TElegantMDI.GetWidth: Integer;
var
  tmp: Integer;
begin
  tmp := 0;
  if not Flat then
    tmp:=tmp+1;

  if ebLeft in EdgeBorders then
    tmp:=tmp+1;

  if ebRight in EdgeBorders then
    tmp:=tmp+1;

  Result := ButtonWidth + tmp*2 + BorderWidth*4;
end;

procedure TElegantMDI.InitRemovable;
begin
//  if BorderWidth=0 then BorderWidth := 3;
  if AutoSize then AutoSize := False;
end;

procedure TElegantMDI.Loaded;
begin
  inherited;
  if Removable then begin
    FRealButtonWidth := ButtonWidth;
    case Align of
      alTop, alBottom: begin
        AutoSize := False;
        Height := Gap;
      end;
      alLeft, alRight: begin
        AutoSize := False;
        Width := Gap;
      end;
    end;
    ButtonWidth := FRealButtonWidth;
  end;
end;

procedure TElegantMDI.SetAbout(const Value: String);
begin
  if FAbout<>Value then RememberMe(Self);
end;

procedure TElegantMDI.SetAnimateSpeed(const Value: Integer);
begin
  if FAnimateSpeed<>Value then begin
    if Value<0 then begin
      Application.MessageBox('Value must be more 0','Error',MB_OK+MB_ICONERROR);
    end
    else
      FAnimateSpeed := Value;
  end;
end;

procedure TElegantMDI.SetDelayOnAppearance(const Value: Integer);
begin
  if FDelayOnAppearance<>Value then begin
    if Value>=0 then
      FDelayOnAppearance := Value
    else begin
      Application.MessageBox('Value can''t be less than 0','Error',MB_OK+MB_ICONERROR);
    end;
  end;
end;

procedure TElegantMDI.SetImageIndexDocument(const Value: Integer);
var
  i: Integer;
begin
  if FImageIndexDocument<>Value then begin
    FImageIndexDocument := Value;
    if ButtonCount>0 then begin
      for i:=0 to ButtonCount-1 do
        Buttons[i].ImageIndex := ImageIndexDocument;
    end;
  end;
end;

procedure TElegantMDI.SetRemovable(const Value: Boolean);
begin
  if FRemovable<>Value then begin
    FRemovable := Value;
  end;
end;

procedure TElegantMDI.WMTimer(var Msg: TMessage);
var
  h: Integer;
  i: Integer;
begin
  KillTimer(Handle,TimerID);
  if (Removable) and (ButtonCount>0) then begin
    ButtonWidth := FRealButtonWidth;
    for i:=0 to ButtonCount-1 do begin
      Buttons[i].Visible := True;
    end;
    case Align of
      alTop, alBottom: begin
        if Height<>GetHeight then begin
          h:=GetHeight;
          if AnimateSpeed>0 then begin
            InitRemovable;
            while Height<h do begin
              Application.ProcessMessages;
              Height := Height + AnimateSpeed;
            end;
          end;
          Height := h;
        end;
      end;
      alLeft, alRight: begin
        h := GetWidth;
        if Width<>h then begin
          if AnimateSpeed>0 then begin
            InitRemovable;
            while Width<h do begin
              Application.ProcessMessages;
              Width := Width + AnimateSpeed;
            end;
          end;
          Width := h;
        end;
      end;
    end;
    ButtonWidth := FRealButtonWidth;
  end;
end;

procedure TElegantMDI.WndClientProc(var Message: TMessage);
begin
  case Message.Msg of
    WM_MDICREATE:
      begin
        Message.Result := CallWindowProc(OldWndClientProc, Form.ClientHandle, Message.Msg, Message.WParam, Message.LParam);

        case Align of
          alTop, alBottom: Wrapable:=False;
          alLeft, alRight: Wrapable:=True;
        end;

        TChild.Create(Message.Result, Self);

        if Removable then begin
          InitRemovable;
//          Perform(CM_MOUSELEAVE,0,0);
        end;

        Exit;
      end;
    WM_DESTROY:
      begin
        SetWindowLong(Form.ClientHandle, GWL_WNDPROC, Longint(OldWndClientProc));
        FreeObjectInstance(InstanceWndClientProc);
      end;
  end;
  Message.Result:= CallWindowProc(OldWndClientProc, Form.ClientHandle, Message.Msg, Message.WParam, Message.LParam);
end;

{ TChild }

procedure TChild.ChildWndProc(var Message: TMessage);
begin
  case Message.Msg of
  WM_GETTEXT:
    begin
      Message.Result:= CallWindowProc(OldChildWndProc, Handle, Message.Msg, Message.WParam, Message.LParam);
      Button.Caption := Copy(PChar(Message.LParam),1,12);
      if Length(PChar(Message.LParam))>12 then begin
        Button.Caption := Button.Caption + ' ...';
        Button.Hint := PChar(Message.LParam);
        Button.ShowHint := True;
      end
      else begin
        Button.Hint := '';
        Button.ShowHint := False;
      end;

      Exit;
    end;
  WM_MDIACTIVATE:
    begin
      if HWND(Message.LParam)=Handle then Button.Down := True;
      if HWND(Message.WParam)=Handle then Button.Down := False;
    end;
  WM_DESTROY:
    begin
      Popup.Free;
      Button.Free;
      SetWindowLong(Handle, GWL_WNDPROC, Longint(OldChildWndProc));
      FreeObjectInstance(InstanceChildWndProc);
    end;
  end;
  Message.Result:= CallWindowProc(OldChildWndProc, Handle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TChild.ClickActBut(Sender: TObject);
begin
  BringWindowToTop(Handle);
end;

procedure TChild.CloseMDIWindow(Sender: TObject);
begin
  SendMessage(Handle,WM_CLOSE,0,0);
end;

procedure TChild.MinimizeMDIWindow(Sender: TObject);
begin
  ShowWindow(Handle,SW_MINIMIZE);
end;

procedure TChild.RestoreMDIWindow(Sender: TObject);
begin
  ShowWindow(Handle,SW_RESTORE);
end;

constructor TChild.Create(AHandle: THandle; AToolBar: TElegantMDI);
var
  i, x: Integer;
  mi: TMenuItem;
begin
  inherited Create;
  Handle := AHandle;
  ToolBar := AToolBar;

  x := 0;
  if ToolBar.ButtonCount>0 then begin
    for i:=0 to ToolBar.ButtonCount-1 do
      case ToolBar.Align of
        alTop, alBottom: begin
          if ToolBar.Buttons[i].Left+ToolBar.Buttons[i].Width>x then
            x := ToolBar.Buttons[i].Left+ToolBar.Buttons[i].Width;
        end;
        alLeft, alRight: begin
          if ToolBar.Buttons[i].Top+ToolBar.Buttons[i].Height>x then
            x := ToolBar.Buttons[i].Top+ToolBar.Buttons[i].Height;
        end;
      end;
  end;

  Button := TToolButton.Create(ToolBar);
  with Button do begin
    Parent := ToolBar;
    Width := ToolBar.ButtonWidth;
    case ToolBar.Align of
      alTop, alBottom: begin
        Left := x + 4;
      end;
      alLeft, alRight: begin
        Top := x + 4;
        Left := 2;
      end;
    end;
    Grouped := True;
    Style := tbsCheck;
    Cursor := crHandPoint;
    ImageIndex := ToolBar.ImageIndexDocument;
    if (ToolBar.Removable) and (ToolBar.Align in [alTop, alBottom]) then
      Visible := False;
  end;
  Button.OnClick := ClickActBut;

  Popup := TPopupMenu.Create(Button);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Activate window';
  mi.OnClick := ClickActBut;
  mi.Bitmap.Handle := LoadBitmap(0, PChar(OBM_CHECK));
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := '-';
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Restore window';
  mi.OnClick := RestoreMDIWindow;
  mi.Bitmap.Handle := LoadBitmap(0, PChar(OBM_RESTORED));
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Minimize window';
  mi.OnClick := MinimizeMDIWindow;
  mi.Bitmap.Handle := LoadBitmap(0, PChar(OBM_REDUCED));
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Maximize window';
  mi.OnClick := MaximizeMDIWindow;
  mi.Bitmap.Handle := LoadBitmap(0, PChar(OBM_ZOOMD));
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := '-';
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Close window';
  mi.OnClick := CloseMDIWindow;
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := '-';
  Popup.Items.Add(mi);

  mi := TMenuItem.Create(Button);
  mi.Caption := 'Cancel';
  Popup.Items.Add(mi);

  Popup.OnPopup := OnPopupMenu;

  Button.PopupMenu := Popup;

  InstanceChildWndProc := MakeObjectInstance(ChildWndProc);
  OldChildWndProc := Pointer(GetWindowLong(Handle,GWL_WNDPROC));
  SetWindowLong(Handle, GWL_WNDPROC, Longint(InstanceChildWndProc));
end;

destructor TChild.Destroy;
begin
  SetWindowLong(Handle, GWL_WNDPROC, Longint(OldChildWndProc));
  FreeObjectInstance(InstanceChildWndProc);
  inherited Destroy;
end;

procedure TChild.OnPopupMenu(Sender: TObject);
begin
  if GetTopWindow(ToolBar.Form.ClientHandle)=Handle then
    Popup.Items[0].Enabled := False
  else
    Popup.Items[0].Enabled := True;

  if IsIconic(Handle) then begin
    Popup.Items[2].Enabled := True;
    Popup.Items[3].Enabled := False;
    Popup.Items[4].Enabled := True;
  end
  else begin
    Popup.Items[3].Enabled := True;
    if IsZoomed(Handle) then begin
      Popup.Items[2].Enabled := True;
      Popup.Items[4].Enabled := False;
    end
    else begin
      Popup.Items[2].Enabled := False;
      Popup.Items[4].Enabled := True;
    end;
  end;
end;

procedure TChild.MaximizeMDIWindow(Sender: TObject);
begin
  ShowWindow(Handle,SW_MAXIMIZE);
end;

end.
